#ifndef GANGU_MQO_MATERIAL_PARSER
#define GANGU_MQO_MATERIAL_PARSER

namespace MQO {

struct MaterialParser
{
	//for functor parser
	typedef py::list result_t;

	typedef boost::shared_ptr<void> pointer_t;
	typedef stored_rule< scanner_t, parser_context<pointer_t> > property_rule;
	typedef std::pair< int, property_rule > kr;
	struct filename_closure:closure< filename_closure, std::string >
	{
		member1 value;
	};

	struct property_closure:closure<property_closure,int,typename property_rule::alias_t>
	{
		member1 key;
		member2 target_p;
	};

	symbols< kr > property_p;
	rule<scanner_t, typename filename_closure::context_t > file_p;

	enum
	{
		//Integral
		ShadingType = 1 << 4,
		VertexColor,
		Projection,

		//Float
		Diffuse = 1 << 5,
		Ambient,
		Emit,
		Specular,
		Hard,

		//String
		ColorMap = 1 << 6,
		NormalMap,
		AlphaMap,
		
		//List
		ProjectLoc = 1 << 7,
		ProjectRot,
		ProjectSize,
		RGB
	};

	MaterialParser();

	std::ptrdiff_t operator()( scanner_t const &scan, py::list &result )const;

	void assignProperty( py::object mat, int key, pointer_t const&val )const;
	void assignIntegral( py::object mat, int key, pointer_t const&val )const;
	void assignFloat( py::object mat, int key, pointer_t const&val )const;
	void assignString( py::object mat, int key, pointer_t const&val )const;
	void assignList( py::object mat, int key, pointer_t const&val )const;
	void createTexture( py::object mat, int key, std::string const &name )const;
};

}//MQO

#endif
