#ifndef GANGU_MQO_OBJECT_PARSER
#define GANGU_MQO_OBJECT_PARSER

namespace MQO {

struct PropValue {
	unsigned patch;
	unsigned segment;
	unsigned visible;
	unsigned mirror;
	unsigned mirror_axis;
	double mirror_dis;
	unsigned facet;
	unsigned shading;
};

struct ObjectParser: parser<ObjectParser>
{
	enum{
		//UInt
		Depth, Folding, Patch, Segment, Visible, Locking, Shading,
		ColorType, Mirror, MirrorAxis, Lathe, LatheAxis, LatheSeg,
		
		//Float
		Facet, MirrorDis,
		
		//List[ Float * 3 ]
		Color, Scale, Rotation, Translation
	};
	typedef std::vector< std::pair< int, py::object > > hierarchy_t;
	typedef std::vector<py::object> group_object_t;

	ObjectParser( double _scale, py::list _materials, group_object_t &_group )
		:scale(_scale), materials(_materials), groupobjs(_group)
	{
	}

	parser_result<ObjectParser, scanner_t>::type
	parse( scanner_t const &scan )const;

	bool parseFace( scanner_t const &scan, py::object nmesh )const;

	bool parseVertex( scanner_t const &scan, py::object nmesh )const;

	bool parseProperty( scanner_t const &scan, py::object nmesh,
		py::object bobject, hierarchy_t const *hierarchy, PropValue *propVal )const;

	void buildHierarchy( hierarchy_t const &hierarchy )const;

	py::list materials;
	double scale;
	group_object_t& groupobjs;
};

}//MQO

#endif
