#include "../include/config.hpp"
#include "../include/blender.hpp"
#include "../include/progress_parser.hpp"
#include "../include/SceneParser.hpp"
#include "../include/MaterialParser.hpp"
#include "../include/ObjectParser.hpp"
#include "../include/MQOParser.hpp"

namespace MQO {

using phoenix::arg1;
using phoenix::bind;

void MQOParser::parse( std::string const &file_name )
{


	typedef file_iterator<char> iterator_t;
	iterator_t first( file_name );

	if( ! first ){
		std::cout << "Unable to OpenFile. " << file_name << std::endl;
		return;
	}
	iterator_t last = first.make_end();
	clock_t sta,fini;
	sta = clock();

	SceneParser scene_p;
	std::cout << "Beginning Conversion." << file_name << std::endl;
	scanner_t scan( first, last );

	rule<scanner_t> head_p =
		progress_p[ bind(&SendProgress)( arg1, "..." ) ]
		>> str_p("Metasequoia Document")
		>> str_p("Format Text Ver 1.0") 
		>> scene_p
		>> ! (str_p("BackImage") >> *( ( anychar_p - ch_p('}') ) ) >> ch_p('}') )
	;

	match<nil_t> hit = head_p.parse(scan);

	if( hit ){
		functor_parser<MaterialParser> material_p;
		match<py::list> mathit = material_p.parse(scan);

		if( mathit ){
			double scale = 1.0;

			if( Blender.GANGu().has_key("Scale") ){
				scale = py::extract<double>( Blender.GANGu()["Scale"] );
			}

			fs::path filename= file_name;
			//py::object group = Blender.Group().attr("New")(filename.leaf());
			ObjectParser::group_object_t groupobjs;

			ObjectParser object_p( scale, mathit.value(), groupobjs );
			hit = object_p.parse(scan);

			int dontgroup = 0;

			if( Blender.GANGu().has_key("DontGroup") ){
				dontgroup = py::extract<int>( Blender.GANGu()["DontGroup"] );
			}

			if (!dontgroup && groupobjs.size() > 1)
			{
				py::object group = Blender.Group().attr("New")(filename.leaf());

				ObjectParser::group_object_t::iterator itr = groupobjs.begin();
				for ( ; itr != groupobjs.end(); itr++ )
					group.attr("objects").attr("link")(*itr);
			}

			/*
			if (group.attr("objects").attr("__len__")()<=1)
			{
				Blender.Group().attr("Unlink")(group);
			}
			*/
		}

		/*
		( *(anychar_p - str_p("Eof"))
			>> str_p("Eof")
			>> end_p
			>> progress_p[ bind(&SendProgress)( arg1, "End" ) ]
			).parse(scan);
		*/
	}
	else{
		SendProgress( 1.0, "Error" );
		std::cout << "s(\n";
		return;
	}
	SendProgress( 1.0, "Done" );

	fini = clock();
	std::cout << "TimeIt " << (double)(fini-sta)/CLOCKS_PER_SEC << "sec" << std::endl;

	//std::string rep( scan.first, scan.first + 20 );
	//std::cout << rep << std::endl;
}

}//MQO

