#include "../include/config.hpp"
#include "../include/blender.hpp"
#include "../include/progress_parser.hpp"
#include "../include/vector_parser.hpp"
#include "../include/return_heap_parser.hpp"
#include "../include/MaterialParser.hpp"

namespace MQO {

using phoenix::arg1;
using phoenix::arg2;
using phoenix::bind;
using phoenix::construct_;
using phoenix::static_cast_;


MaterialParser::MaterialParser()
{
	file_p = ch_p('"')
		>> ( *( anychar_p - ch_p('"') ) )[ file_p.value = construct_<std::string>(arg1, arg2)]
		>> ch_p('"')
	;

	property_p.add
		("shader",		kr( ShadingType, heap_p(uint_p) ))
		("vcol",		kr( VertexColor, heap_p(uint_p) ))
		("proj_type",	kr( Projection, heap_p(uint_p) ))
		("dif",			kr( Diffuse, heap_p(real_p)))
		("amb",			kr( Ambient, heap_p(real_p)))
		("emi",			kr( Emit, heap_p(real_p)))
		("spc",			kr( Specular, heap_p(real_p)))
		("power",		kr( Hard, heap_p(real_p)))
		("tex",			kr( ColorMap, heap_p(file_p)))
		("bump",		kr( NormalMap, heap_p(file_p)))
		("alpha",		kr( AlphaMap, heap_p(file_p)))
		("aplane",		kr( AlphaMap, heap_p(file_p)))
		("proj_pos",	kr( ProjectLoc, heap_p( vector_p<3>(real_p) )))
		("proj_angle",	kr( ProjectRot, heap_p( vector_p<3>(real_p) )))
		("proj_scale",	kr( ProjectSize, heap_p( vector_p<3>(real_p) )))
		("col",			kr( RGB, heap_p( vector_p<4>(real_p) ) ))
	;
}

std::ptrdiff_t
MaterialParser::operator()( scanner_t const &scan, py::list &result )const
{
	if( scan.at_end()){
		return -1;
	}

	match<nil_t> hit = ( str_p("Material") >> uint_p >> ch_p('{') ).parse( scan );
	if( !hit ){
		return -1;
	}

	clock_t sta,fini;
	sta = clock();

	scanner_t::iterator_t save = scan.first;
	py::object current;

	rule<scanner_t, typename property_closure::context_t > mat_p;
	mat_p =	*(
		property_p
		[
			mat_p.key = bind(&kr::first)(static_cast_<kr&>(arg1)),
			mat_p.target_p = bind(&kr::second)(static_cast_<kr&>(arg1))
		]
		>> ch_p('(')
		>> lazy_p( mat_p.target_p )
		[
			bind(&MaterialParser::assignProperty)( *this, boost::ref(current), mat_p.key, arg1 )
		]
		>> ch_p(')')
	);

	for(;;){
		std::string name;
		match<nil_t> hit = file_p[ assign_a(name) ].parse(scan);

		if( !hit )break;

		current = Blender.Material().attr("New")( name );
		std::cout << "Creating Material -" << name << std::endl;;
		hit = mat_p.parse( scan );

		if( hit ){
			result.append( current );
		}
		else{
			break;
		}
	}
	if( save == scan.first ){
		std::cout << "Parse Error in Material Parser" << std::endl;
		return -1;
	}
	fini = clock();
	std::cout << "Material Convert Time " << (double)(fini-sta)/CLOCKS_PER_SEC << "sec\n\n" << std::endl;

	ch_p('}').parse(scan);
	return (std::ptrdiff_t)(scan.first - save);
}

}//MQO

