#include "../include/config.hpp"
#include "../include/blender.hpp"
#include "../include/progress_parser.hpp"
#include "../include/binary_parser.hpp"
#include "../include/vector_parser.hpp"
#include "../include/ObjectParser.hpp"

namespace MQO {

using phoenix::bind;
using phoenix::arg1;

struct AppendVertex
{
	typedef void result_type;

	AppendVertex( double _scale )
		:scale(_scale)
	{
		vert = Blender.NMesh().attr("Vert");
	}

	void operator()( py::object nmesh, boost::array<float,3> const &co )const
	{
		nmesh.attr("verts").attr("append")( vert(
			co[0] * scale, -co[2]* scale, co[1] * scale )
		);
	}

	double scale;
	py::object vert;
};


bool ObjectParser::parseVertex( scanner_t const &scan, py::object nmesh )const
{
	AppendVertex av( scale );
	unsigned int count = 0;
	real_parser<float, real_parser_policies<float> > float_p;
	binary_parser<float> bin_p;

	return (
		progress_p[ bind(&SendProgress)( arg1, "..." ) ]
		>>!( str_p("vertex") >> uint_p[ assign_a(count) ] >> ch_p('{')
		>> repeat_p(boost::ref(count))[
			vector_p<3>( float_p )[bind(av)( nmesh, arg1)]
		]
		>> ch_p('}')
		>> !( str_p("vertexattr") >> ch_p('{')
			>> !( str_p("weit") >> ch_p('{') >> *( anychar_p - ch_p('}') ) >> ch_p('}') )
			>> !( str_p("color") >> ch_p('{') >> *( anychar_p - ch_p('}')) >> ch_p('}') )
			>> ch_p('}')
			)
			)
		>> !( str_p("BVertex") >> uint_p[ assign_a(count) ] >> ch_p('{')
			>> str_p("Vector") >> int_p >> ch_p('[') >> int_p >> ch_p(']')
			>> lexeme_d[
				repeat_p(boost::ref(count))
				[
					vector_p<3>( bin_p )
					[ bind(av)( nmesh, arg1) ]
				]
			]
			>> !( str_p("weit") >> ch_p('{') >> *( anychar_p - ch_p('}') ) >> ch_p('}') )
			>> !( str_p("color") >> ch_p('{') >> *( anychar_p - ch_p('}')) >> ch_p('}') )
			>> ch_p('}')
	) ).parse(scan).operator_bool();
}

}