#include "../include/config.hpp"
#include "../include/blender.hpp"
#include "../include/pylist_inserter_parser.hpp"
#include "../include/progress_parser.hpp"
#include "../include/binary_parser.hpp"
#include "../include/ObjectParser.hpp"

namespace MQO {

parser_result<ObjectParser, scanner_t>::type
ObjectParser::parse( scanner_t const &scan )const
{
	if( scan.at_end() ){
		return scan.no_match();
	}
	hierarchy_t hierarchy;
	scanner_t::iterator_t save = scan.first;

	std::cout << "Beginning Object Stage\n";

	while(1){
		std::string name;
		match<nil_t> hit = (
			str_p("Object") >> ch_p('"')
			>> ( *(anychar_p - ch_p('"')) )[assign_a( name )]
			>> ch_p('"') >> ch_p('{')
		).parse( scan );

		if( !hit ){	break;}

		std::cout << "Parsing Object [" << name << "]" << std::endl;

		py::object bobject = Blender.Object().attr("New")("Mesh", name);
		py::object nmesh = Blender.NMesh().attr("New")(name);

		PropValue propVal;

		if( ! parseProperty( scan, nmesh, bobject, &hierarchy, &propVal ) ){
			break;
		}
		if( ! parseVertex( scan, nmesh ) ){
			break;
		}

		std::cout
			<< "\tVertices "
			<< py::extract<int>( nmesh.attr("verts").attr("__len__")() )
			<< " Converted" << std::endl;

		if( ! parseFace( scan, nmesh ) ){
			break;
		}

		std::cout
			<< "\tFaces "
			<< py::extract<int>( nmesh.attr("faces").attr("__len__")() )
			<< " Converted" << std::endl;

		if (propVal.visible == 0){
				bobject.attr("restrictDisplay") = (py::object)true;
		}

		if (propVal.mirror){
				//add 'mirror' modifier
				py::object modSeq = bobject.attr("modifiers");
				py::object mirror = Blender.Modifier().attr("Types").attr("MIRROR");
				py::object mod = modSeq.attr("append")(mirror);

				char *str_axis[] = { "AXIS_X", "AXIS_Z", "AXIS_Y"};

				if (propVal.mirror_axis==0) propVal.mirror_axis=1;

				for (unsigned int i = 0, axis = propVal.mirror_axis; i < 3; i++, axis>>=1)
				{
					py::object mirroraxis = Blender.Modifier().attr("Settings").attr(str_axis[i]);
					mod[mirroraxis] = axis & 1;
				}

				/*
				if (propVal.mirror == 2){
					py::object mirrordis = Blender.Modifier().attr("Settings").attr("LIMIT");
					mod[mirrordis] = propVal.mirror_dis * scale;
				}
				*/
		}

		if (propVal.patch){
				//add 'subsurf' modifier
				py::object modSeq = bobject.attr("modifiers");
				py::object subsurf = Blender.Modifier().attr("Types").attr("SUBSURF");
				py::object mod = modSeq.attr("append")(subsurf);

				if (propVal.patch == 3){	//Catmull-Clark
					py::object sslevel = Blender.Modifier().attr("Settings").attr("LEVELS");
					mod[sslevel] = propVal.segment;
				}

				//set smooth
				py::object faces = nmesh.attr("faces");
				for(int i=0; i<nmesh.attr("faces").attr("__len__")();i++)
					faces[i].attr("smooth") = (py::object)true;
		}

		if (propVal.shading){
				unsigned maxangle = propVal.facet > 80 ? 80 : propVal.facet;
				nmesh.attr("setMaxSmoothAngle")(maxangle);

				nmesh.attr("setMode")("AutoSmooth");

				if (!propVal.patch){
					py::object faces = nmesh.attr("faces");
					for(int i=0; i<nmesh.attr("faces").attr("__len__")();i++)
						faces[i].attr("smooth") = (py::object)true;
				}
		}

		nmesh.attr("update")();
		bobject.attr("link")( nmesh );
		bobject.attr("sel") = (py::object)1;

		Blender.Scene().attr("GetCurrent")().attr("objects").attr("link")( bobject );
		//group.attr("objects").attr("link")(bobject);
		groupobjs.push_back(bobject);

		ch_p('}').parse(scan);
	}

	if( save != scan.first && hierarchy.size() ){
		buildHierarchy( hierarchy );
		return scan.create_match( scan.first - save, nil_t(), scan.first, scan.first );
	}
	return scan.no_match();
}

}//MQO